# _*_ coding: utf-8 _*_

'''
自动站资料客观分析程序
朱文剑

'''

# %%
from metpy.interpolate import interpolate_to_grid
import nmc_met_io.retrieve_micaps_server as mserver
import pandas as pd
import matplotlib.pyplot as plt
import xarray as xr
import numpy as np
from collections import OrderedDict
from metpy.units import units
from metpy.calc import wind_components,vorticity,divergence,lat_lon_grid_deltas
import os
import gc
from datetime import datetime,timedelta
import matplotlib
matplotlib.use('Agg')


def remove_nan_observations(x, y, z):
    r"""Remove all x, y, and z where z is nan.

    Will not destroy original values.

    Parameters
    ----------
    x: array_like
        x coordinate
    y: array_like
        y coordinate
    z: array_like
        observation value

    Returns
    -------
    x, y, z
        List of coordinate observation pairs without
        nan valued observations.

    """
    x_ = x[~np.isnan(z)]
    y_ = y[~np.isnan(z)]
    z_ = z[~np.isnan(z)]

    return x_, y_, z_
    
def remove_repeat_coordinates(x, y, z):
    r"""Remove all x, y, and z where (x,y) is repeated and keep the first occurrence only.

    Will not destroy original values.

    Parameters
    ----------
    x: array_like
        x coordinate
    y: array_like
        y coordinate
    z: array_like
        observation value

    Returns
    -------
    x, y, z
        List of coordinate observation pairs without
        repeated coordinates.

    """
    coords = []
    variable = []

    for (x_, y_, t_) in zip(x, y, z):
        if (x_, y_) not in coords:
            coords.append((x_, y_))
            variable.append(t_)

    coords = np.array(coords)

    x_ = coords[:, 0]
    y_ = coords[:, 1]

    z_ = np.array(variable)

    return x_, y_, z_

def get_contour_verts(cn):
    contours = []
    # for each contour line
    for cc in cn.collections:
        paths = []
        # for each separate section of the contour line
        for pp in cc.get_paths():
            paths.append(pp.vertices)
        contours.append(paths)

    return contours

# 客观分析类
# 参考metpy等诊断分析库
class Object_Analyst:
    def __init__(self) -> None:
        self.debug_level = 0
        self.minlon = 100
        self.maxlon = 120
        self.minlat = 30
        self.maxlat = 45
        self.reso=0.025

    # 设置北京时还是世界时
    def set_time_type(self,ttype):
        self.time_type = ttype

    def set_reso(self,reso):
        self.reso = reso

    def set_debug(self,level):
        self.debug_level=level

    # 设置要分析的经纬度范围
    def set_boundary(self,minlon,maxlon,minlat,maxlat):
        self.minlon = minlon
        self.maxlon = maxlon
        self.minlat = minlat
        self.maxlat = maxlat
        pass
    
    # 设置当前时刻
    def set_time(self,tstr='20220511141000'):
        self.time_str=tstr

    # 设置X坐标
    def set_xcord(self,data_array):
        self.xcord = data_array
    

    # 设置Y坐标
    def set_ycord(self,data_array):
        self.ycord = data_array

    # 设置要客观分析的变量
    def set_vardata(self,data_array):
        self.vardata = data_array

    # 判断一个点是否在边界内
    def is_in_boundary(self,point_lon,point_lat):
        if point_lon < self.minlon \
                or point_lon > self.maxlon \
                    or point_lat < self.minlat \
                        or point_lat > self.maxlat:
                            return False
        else:
            return True

    # 针对从天擎下载的5分钟资料做客观分析，是从csv文件中读取的datafreame格式
    def do_oa_csv_df(self,newdata=None,vartype='温度'):

        if newdata is None:
            print('newdata is None!')
            return None
        
        
        if self.debug_level > 0:
            print(newdata.columns)
        
        validindex=[]
        for nn in range(newdata.shape[0]):
            if not self.is_in_boundary(newdata['Lon'][nn],newdata['Lat'][nn]):
                continue
            else:
                validindex.append(nn)
            

        if len(validindex) <=10:
            print('valid data less than 10, skip!')
            return None
        
        # print(validindex)        

        defaultvalue=999999
        # 注意，务必将缺测值设置为nan
        latlist = newdata['Lat'][validindex].values
        lonlist = newdata['Lon'][validindex].values
        templist = newdata['TEM'][validindex].values
        rhlist = newdata['RHU'][validindex].values
        dtlist = newdata['DPT'][validindex].values
        sprslist = newdata['PRS_Sea'][validindex].values

        self.set_boundary(np.min(lonlist),np.max(lonlist),np.min(latlist),np.max(latlist))
        if vartype.find('温度')>=0:
            flag = templist < defaultvalue  
            lonlist = lonlist[flag]
            latlist = latlist[flag]
            templist = templist[flag]

            params={}
            params['in_lon'] = lonlist
            params['in_lat'] = latlist
            params['in_data'] = templist
            params['out_varname'] = 't2m'
            params['out_long_name'] = 'surface temperature objective analyse'
            params['out_short_name'] = 'oa_t'
            params['out_units'] = 'degC'
             
            params['tipname'] = '温度'
            outdata = self.do_oa_base(params)

        elif vartype.find('露点')>=0:
            flag = dtlist < defaultvalue  
            lonlist = lonlist[flag]
            latlist = latlist[flag]
            dtlist = dtlist[flag]

            params={}
            params['in_lon'] = lonlist
            params['in_lat'] = latlist
            params['in_data'] = dtlist
            params['out_varname'] = 'td2m'
            params['out_long_name'] = 'surface dewpoint temperature objective analyse'
            params['out_short_name'] = 'oa_td'
            params['out_units'] = 'degC'
             
            params['tipname'] = '露点温度'
            outdata = self.do_oa_base(params)
            
        elif vartype.find('湿度')>=0:
            flag = rhlist < defaultvalue  
            lonlist = lonlist[flag]
            latlist = latlist[flag]
            rhlist = rhlist[flag]

            params={}
            params['in_lon'] = lonlist
            params['in_lat'] = latlist
            params['in_data'] = rhlist
            params['out_varname'] = 'rh2m'
            params['out_long_name'] = 'surface relative humidility objective analyse'
            params['out_short_name'] = 'oa_rh'
            params['out_units'] = '%'
             
            params['tipname'] = '相对湿度'
            outdata = self.do_oa_base(params)

        elif vartype.find('气压')>=0:
            flag = sprslist < defaultvalue  
            lonlist = lonlist[flag]
            latlist = latlist[flag]
            sprslist = sprslist[flag]

            params={}
            params['in_lon'] = lonlist
            params['in_lat'] = latlist
            params['in_data'] = sprslist
            params['out_varname'] = 'sprs2m'
            params['out_long_name'] = 'sea pressure objective analyse'
            params['out_short_name'] = 'oa_sprs'
            params['out_units'] = 'hPa'
             
            params['tipname'] = '海平面气压'
            outdata = self.do_oa_base(params)

        return outdata
    
    # 针对从天擎下载的5分钟资料做客观分析，是存储为csv格式的文件
    def do_oa_csv(self,filename,vartype='温度'):

        if not os.path.exists(filename):
            print(filename + ' not exists!')
            return None
        newdata = pd.read_csv(filename,encoding='gbk')
        
        if self.debug_level > 0:
            print(newdata.columns)
        
        validindex=[]
        for nn in range(newdata.shape[0]):
            if not self.is_in_boundary(newdata['Lon'][nn],newdata['Lat'][nn]):
                continue
            else:
                validindex.append(nn)
            

        if len(validindex) <=10:
            print('valid data less than 10, skip!')
            return None
        
        # print(validindex)        

        defaultvalue=999999
        tipname = ''
        # 注意，务必将缺测值设置为nan
        if vartype.find('温度')>=0:
            varname = 'TEM'
            tipname = '温度'
        elif vartype.find('湿度')>=0:
            varname = 'RHU'
            tipname = '相对湿度'
        elif vartype.find('气压')>=0:
            varname = 'PRS_Sea'
            tipname = '海平面气压'
        elif vartype.find('露点')>=0:
            varname = 'DPT'
            tipname = '露点温度'
        elif vartype.find('变温')>=0:
            for var in newdata.columns:
                if var.find('TEM_delta')>=0:
                    varname = var
                    tipname = '%s小时变温'%var.split('_')[-1][0]
                    break
        elif vartype.find('变压')>=0:           
            for var in newdata.columns:
                if var.find('PRS_Sea_delta')>=0:
                    varname = var
                    tipname = '%s小时变压'%var.split('_')[-1][0]
                    break


        latlist = newdata['Lat'][validindex].values
        lonlist = newdata['Lon'][validindex].values
        # templist = newdata['TEM'][validindex].values
        # rhlist = newdata['RHU'][validindex].values
        # dtlist = newdata['DPT'][validindex].values
        # sprslist = newdata['PRS_Sea'][validindex].values
        datalist = newdata[varname][validindex].values

        self.set_boundary(np.min(lonlist),np.max(lonlist),np.min(latlist),np.max(latlist))
        flag = datalist < defaultvalue  
        lonlist = lonlist[flag]
        latlist = latlist[flag]
        datalist = datalist[flag]
        params={}
        params['in_lon'] = lonlist
        params['in_lat'] = latlist
        params['in_data'] = datalist
        params['tipname'] = tipname
        if vartype.find('温度')>=0:
            params['out_varname'] = 't2m'
            params['out_long_name'] = 'surface temperature objective analyse'
            params['out_short_name'] = 'oa_t'
            params['out_units'] = 'degC'
            outdata = self.do_oa_base(params)
        elif vartype.find('露点')>=0:
            params['out_varname'] = 'td2m'
            params['out_long_name'] = 'surface dewpoint temperature objective analyse'
            params['out_short_name'] = 'oa_td'
            params['out_units'] = 'degC' 
            outdata = self.do_oa_base(params)
        elif vartype.find('湿度')>=0:
            params['out_varname'] = 'rh2m'
            params['out_long_name'] = 'surface relative humidility objective analyse'
            params['out_short_name'] = 'oa_rh'
            params['out_units'] = '%'
            outdata = self.do_oa_base(params)
        elif vartype.find('气压')>=0:
            params['out_varname'] = 'sprs2m'
            params['out_long_name'] = 'sea pressure objective analyse'
            params['out_short_name'] = 'oa_sprs'
            params['out_units'] = 'hPa'
            outdata = self.do_oa_base(params)
        elif vartype.find('变温')>=0:
            params['out_varname'] = 't2m_delta_' + varname.split('_')[-1]
            params['out_long_name'] = 'surface temperature delta objective analyse'
            params['out_short_name'] = 'oa_t_delta' + varname.split('_')[-1]
            params['out_units'] = 'degC'
            outdata = self.do_oa_base(params)
        elif vartype.find('变压')>=0:
            params['out_varname'] = 'sprs2m_delta_' + varname.split('_')[-1]
            params['out_long_name'] = 'sea pressure delta objective analyse'
            params['out_short_name'] = 'oa_sprs_delta' + varname.split('_')[-1]
            params['out_units'] = 'hPa'
            outdata = self.do_oa_base(params)
        return outdata

    # 进行客观分析的基础函数
    def do_oa_base(self,params):
    

        if not 'in_lon' in params.keys():
            print('key "in_lon" should be set')
            return False

        if not 'in_lat' in params.keys():
            print('key "in_lat" should be set')
            return False
        
        if not 'in_data' in params.keys():
            print('key "in_data" should be set')
            return False

        if not 'out_varname' in params.keys():
            print('key "out_varname" should be set')
            return False

        if not 'out_long_name' in params.keys():
            print('key "out_long_name" should be set')
            return False
        
        if not 'out_short_name' in params.keys():
            print('key "out_short_name" should be set')
            return False

        if not 'out_units' in params.keys():
            print('key "out_units" should be set')
            return False

        lonlist = params['in_lon']
        latlist = params['in_lat']
        data = params['in_data']
        x_masked, y_masked, data = remove_nan_observations(lonlist, latlist, data)
        x_masked, y_masked, data = remove_repeat_coordinates(x_masked, y_masked, data)

        # 4sigma
        def four_sigma(df):
            mean=df.data.mean()
            std=df.data.std()
            upper_limit=mean+4*std
            lower_limit=mean-4*std
            df['anomaly']=df.data.apply(lambda x: 1 if (x>upper_limit ) or (x<lower_limit) else 0)
            return df

        # 根据百分位进行异常值检查
        tmpdata = pd.DataFrame(data,columns=['data'])
        df1 = four_sigma(tmpdata)
        # df1[df1['anomaly']==1]

        x_masked = x_masked[df1['anomaly']==0]
        y_masked = y_masked[df1['anomaly']==0]
        data = data[df1['anomaly']==0]

        if self.debug_level > 0:
            print('read data over!')

        # self.set_reso(0.05)
        # 温度插值
        # hreso = 0.05

        gx, gy, gd = interpolate_to_grid(x_masked, y_masked, data,
                                                    interp_type='cressman',hres=self.reso,
                                                    minimum_neighbors=1,
                                                    )


        # 将客观分析结果在x和y方向调整坐标
        gd = gd.T
        gx = gx.T
        gy = gy.T

        # 构建xarray，并返回

        # define coordinates
        # time_coord = ('time', redic['time'][0])
        lon_coord = ('lon', gx[:,0], {
            'long_name':'longitude', 'units':'degrees_east', '_CoordinateAxisType':'Lon'})
        lat_coord = ('lat', gy[0,:], {
            'long_name':'latitude', 'units':'degrees_north', '_CoordinateAxisType':'Lat'})


        # create xarray
        varattrs={'long_name': params['out_long_name'], 'short_name': params['out_short_name'], 'units': params['out_units']}
        
     
        result = xr.Dataset({params['out_varname']:(['lon', 'lat'], gd, varattrs),
                             },
                            coords={ 'lon':lon_coord, 'lat':lat_coord })

        # add attributes
        result.attrs['Conventions'] = "CF-1.6"
        result.attrs['time'] = self.time_str# np.datetime64(datetime.strptime(self.time_str,'%Y%m%d%H%M%S',))
        result.attrs['sta_maxvalue'] = np.max(data)
        result.attrs['sta_minvalue'] = np.min(data)
        if params['out_varname'] == 'sprs2m':
            result.attrs['sta_maxvalue'] = (np.max(data) + 2.5)//2.5*2.5
            result.attrs['sta_minvalue'] = (np.min(data) + 2.5)//2.5*2.5
        result.attrs['varname'] = params['out_varname']
        result.attrs['tipname'] = params['tipname']
        return result

    # 针对mdfs的5分钟资料做客观分析
    def do_oa_mdfs(self,filename,vartype='温度'):
        if not os.path.exists(filename):
            print(filename + ' not exists!')
            return None

        redic = mserver.get_stadata_from_mdfs(filename)
        if self.debug_level > 0:
            print(redic.columns)
        
        newdata = redic[['ID','lon','lat','温度','露点温度','相对湿度','平均风速_2分钟','平均风向_2分钟']]
        validindex=[]
        for nn in range(newdata.shape[0]):
            if not self.is_in_boundary(newdata['lon'][nn],newdata['lat'][nn]):
                continue
            else:
                validindex.append(nn)
        # print(validindex)        

        # 注意，务必将缺测值设置为nan
        latlist = newdata['lat'][validindex].values
        lonlist = newdata['lon'][validindex].values
        templist = newdata['温度'][validindex].values
        rhlist = newdata['相对湿度'][validindex].values
        dtlist = newdata['露点温度'][validindex].values
        windspd = newdata['平均风速_2分钟'][validindex].values
        winddir = newdata['平均风向_2分钟'][validindex].values

        if vartype.find('温度')>=0:
            params={}
            params['in_lon'] = lonlist
            params['in_lat'] = latlist
            params['in_data'] = templist
            params['out_varname'] = 't2m'
            params['out_long_name'] = 'surface temperature objective analyse'
            params['out_short_name'] = 'oa_t'
            params['out_units'] = 'degC'
             
            outdata = self.do_oa_base(params)
        elif vartype.find('露点')>=0:
            params={}
            params['in_lon'] = lonlist
            params['in_lat'] = latlist
            params['in_data'] = dtlist
            params['out_varname'] = 'td2m'
            params['out_long_name'] = 'surface dewpoint temperature objective analyse'
            params['out_short_name'] = 'oa_td'
            params['out_units'] = 'degC'
             
            outdata = self.do_oa_base(params)
        elif vartype.find('湿度')>=0:
            params={}
            params['in_lon'] = lonlist
            params['in_lat'] = latlist
            params['in_data'] = rhlist
            params['out_varname'] = 'rh2m'
            params['out_long_name'] = 'surface relative humidility objective analyse'
            params['out_short_name'] = 'oa_rh'
            params['out_units'] = '%'
             
            outdata = self.do_oa_base(params)

        # # wind
        # wind_speed = (windspd * units('m/s'))
        # wind_dir = winddir * units.degree
        # good_indices = np.where((~np.isnan(wind_dir)) & (~np.isnan(wind_speed)))

        # x_masked = lonlist[good_indices]
        # y_masked = latlist[good_indices]
        # wind_speed = wind_speed[good_indices]
        # wind_dir = wind_dir[good_indices]
        # u, v = wind_components(wind_speed, wind_dir)

        # # uwind
        # params['in_lon'] = x_masked
        # params['in_lat'] = y_masked
        # params['in_data'] = u.magnitude
        # params['out_varname'] = 'u10m'
        # params['out_long_name'] = '10m uwind objective analyse'
        # params['out_short_name'] = 'oa_u10m'
        # params['out_units'] = 'm/s'
        #  
        # u10m = self.do_oa_base(params)

        # # vwind
        # params['in_lon'] = x_masked
        # params['in_lat'] = y_masked
        # params['in_data'] = v.magnitude
        # params['out_varname'] = 'v10m'
        # params['out_long_name'] = '10m vwind objective analyse'
        # params['out_short_name'] = 'oa_v10m'
        # params['out_units'] = 'm/s'
        #  
        # v10m = self.do_oa_base(params)

        # digdata = self.calc_vor_div(wind_speed,wind_dir,lonlist,latlist)

        # 
        
        return outdata
    
    def calc_vor_div(self,wind_speed,wind_dir,lonlist,latlist):
        
        # 风场插值

        good_indices = np.where((~np.isnan(wind_dir)) & (~np.isnan(wind_speed)))

        x_masked = lonlist[good_indices]
        y_masked = latlist[good_indices]
        wind_speed = wind_speed[good_indices]
        wind_dir = wind_dir[good_indices]
        u, v = wind_components(wind_speed, wind_dir)
        windgridx, windgridy, uwind = interpolate_to_grid(x_masked, y_masked, np.array(u),interp_type='cressman', hres=self.reso)

        _, _, vwind = interpolate_to_grid(x_masked, y_masked, np.array(v), interp_type='cressman',hres=self.reso)
        if self.debug_level > 0:
            print('interpolate done!')

        # 计算涡度散度
        dx, dy = lat_lon_grid_deltas(longitude=windgridx, latitude=windgridy)
        vtx=vorticity(u=uwind*units.meter/units.seconds ,v=vwind*units.meter/units.seconds,dx=dx,dy=dy)
        div=divergence(u=uwind*units.meter/units.seconds ,v=vwind*units.meter/units.seconds,dx=dx,dy=dy)

        # 乘上1e5，方便显示
        vtx = vtx * 1e5
        div = div * 1e5
        # print(vtx.shape)
        # print(div.shape)
        if self.debug_level > 0:
            print('涡度和散度计算完毕！')

        # 构建xarray，并返回

        # define coordinates
        # time_coord = ('time', redic['time'][0])
        lon_coord = ('lon', windgridx[0,:], {
            'long_name':'longitude', 'units':'degrees_east', '_CoordinateAxisType':'Lon'})
        lat_coord = ('lat', windgridy[:,0], {
            'long_name':'latitude', 'units':'degrees_north', '_CoordinateAxisType':'Lat'})


        # create xarray
        varattrs_vtx10m={'long_name': 'surface vortex ', 'short_name': 'oa_vtx_10m', 'units': '1e-5*1/s'}
        varattrs_div10m={'long_name': 'surface divergence ', 'short_name': 'oa_div_10m', 'units': '1e-5*1/s'}
     
        result = xr.Dataset({
                             'vtx_10m':(['lat', 'lon'], vtx.magnitude, varattrs_vtx10m),
                             'div_10m':(['lat', 'lon'], div.magnitude, varattrs_div10m),
                             },
                            coords={ 'lat':lat_coord, 'lon':lon_coord})

        # add attributes
        result.attrs['Conventions'] = "CF-1.6"

        # 对计算结果进行平滑处理
        vtx_smooth=result.vtx_10m.rolling(lon=5, lat=5, min_periods=1, center=True).mean()
        result.vtx_10m.data = vtx_smooth.values

        div_smooth=result.div_10m.rolling(lon=5, lat=5, min_periods=1, center=True).mean()
        result.div_10m.data = div_smooth.values
        
        return result


    

if __name__ == '__main__':

    filepath = '/home/wjzhu/OneDrive/PythonCode/MyWork/metradar/testdata/aws/'
    # filename = 'PLOT_5MIN_20220511141000.000'
    filename = 'surface_aws_20230409_0818.csv'
    startlon = 106
    endlon = 135
    startlat = 18
    endlat = 28

    oa_class = Object_Analyst()
    oa_class.set_debug(1)
    oa_class.set_reso(0.025)
    oa_class.set_boundary(startlon,endlon,startlat,endlat)
    
    # oa_class.set_time('20220511141000')
    tstr = filename[12:20]+filename[21:25]
    oa_class.set_time(tstr)
    # result = oa_class.do_oa_mdfs(filepath + os.sep + filename,vartype='温度')
    result = oa_class.do_oa_csv(filepath + os.sep + filename,vartype='温度')
    
    print('done! ')


    if not result is None:
        # cflag = (abs(result.t2m)<20) & (abs(result.t2m)>3)
        result.t2m.plot.contourf(levels=15, add_colorbar=True)
        plt.title('t2m')
        plt.show()
  
        kk=0

    # if not result is None:
    #     cflag = (abs(result.div_10m)<20) & (abs(result.div_10m)>3)
    #     result.div_10m.where(cflag).plot.contour(levels=15, add_colorbar=True)
    #     plt.title('divergence')
    #     plt.show()
    #     kk=0


    # oa_class = Object_Analyst()
    # oa_class.set_reso(0.01)
    # params={}
    # params['in_lon'] = aws_lons
    # params['in_lat'] = aws_lats
    # params['in_data'] = aws_tem
    # params['out_varname'] = 't2m'
    # params['out_long_name'] = 'surface temperature objective analyse'
    # params['out_short_name'] = 'oa_t'
    # params['out_units'] = 'degC'
    
    # t2m = oa_class.do_oa_base(params)
    

    

    
    # %%



'''
测试将雷达拼图输出成pgmb格式

'''

# %%
import matplotlib
matplotlib.use('Agg')  # 关键：禁用交互式后端
from metradar.io.pgmb_io import pgmb_write
import os
from metradar.io.read_new_mosaic_func import decode_mosaic
import xarray as xr
import numpy as np
import gzip


def zip_file(path, new_path):
    with open(new_path, 'wb') as wf:
        with open(path, 'rb') as rf:
            data = rf.read()
            # 压缩数据
            data_comp = gzip.compress(data,compresslevel=5)
        wf.write(data_comp)
 
 
filepath = '/mnt/e/metradar_test/pysteps_data/mosaic_bin/2025/07/02/RADA_L3_MST_CREF_QC/'
for filename in os.listdir(filepath):
    
    data = decode_mosaic(filepath,filename)
    # filename = 'ACHN.CREF000.20230421.180009.nc'
    obstimestr = filename.split('_')[9]+filename.split('_')[10][0:4].split('.')[0]
    outname = obstimestr + '_fmi.radar.composite.lowest_FIN_SUOMI1.pgm'
    outpath = '/mnt/e/metradar_test/pysteps_data/radar/fmi/%s'%filename.split('_')[9]
    if not os.path.exists(outpath):
        os.makedirs(outpath)

    file_name = outpath + os.sep + outname

    data = data.sel(lat=slice(33, 37), lon=slice(109, 117))

    width = data.CREF.shape[1]
    height = data.CREF.shape[0]
    maxval = 255
    gray = data.CREF*2+66

    aa = np.isnan(gray.data)
    gray.data[aa]=255
    cref = np.flipud(gray.data.astype(np.uint8))
    params={}
    params['obstimestr'] = obstimestr
    params['left_lon'] = 113
    params['right_lon'] = 118
    params['bottom_lat'] = 37
    params['upper_lat'] = 41

    pgmb_write ( file_name, params, width, height, maxval, cref )

    # gzip file

    # zip_file(file_name, file_name + '.gz')


    print('write pgmb file %s successfully!'%(file_name))
    # break


# _*_ coding: utf-8 _*_

'''
构建时间序列图的框架程序
'''

from nmc_met_io.retrieve_cmadaas import cmadaas_obs_by_time_range_and_id
from nmc_met_io.retrieve_cmadaas import cmadaas_sounding_by_time
from nmc_met_io.retrieve_cmadaas import cmadaas_radar_level2_by_timerange_and_id

import pandas as pd
import geo_transforms_pyart as geotrans
import math
import numpy as np
import os
import configparser
# from make_mosaic_mp_archive import MAKE_RADAR_MOSAIC
from metradar.project.make_mosaic.make_mosaic_func import MAKE_RADAR_MOSAIC
from multiprocessing import freeze_support
from datetime import datetime,timedelta
import xarray as xr
import numpy.ma as MA
import matplotlib.pyplot as plt
from metradar.util.parse_pal import parse_pro
from matplotlib.ticker import (MultipleLocator)
from matplotlib import font_manager
import matplotlib
from metradar.util.comm_func import geopotential_to_height
matplotlib.use('agg')

from pathlib import Path
from metradar.config import CONFIG

# 资源文件路径
RESOURCES_PATH = CONFIG.get('SETTING','RESOURCES_PATH')
FONT_FILE = RESOURCES_PATH + '/fonts/YaHeiConsolasHybrid_1.12.ttf'
RADAR_FILE = RESOURCES_PATH + '/stations/radars.csv'
COLOR_FILE = RESOURCES_PATH + '/gr2_colors/default_BR_PUP1.pal'

# 获取当前脚本所在目录
SCRIPT_DIR = Path(__file__).resolve().parent
print("当前脚本路径：",SCRIPT_DIR)

# 一键生成VPR数据和自动站数据，并绘制图形
#数据来源：天擎

class CONSTRUCT_VPR:

    # sub function for reading config file
    def ConfigFetchError(Exception):
        pass

    def _get_config_from_rcfile(self,rcfile):
        """
        Get configure information from config_dk_met_io.ini file.
        """
        
        rc = rcfile
        print(rc)
        if not os.path.exists(rc):
            print(rc + ' not exist')
            return None
        try:
            config = configparser.ConfigParser()
            config.read(rc,encoding='UTF-8')
        except IOError as e:
            raise self.ConfigFetchError(str(e))
        except Exception as e:
            raise self.ConfigFetchError(str(e))

        return config
    
    def __init__(self,inifilepath) -> None:
        pass
        config = self._get_config_from_rcfile(inifilepath)
        self.outpath = config['SETTINGS']['root_outpath']
        self.outpath_fmt = self.outpath + os.sep + config['SETTINGS']['radar_basedata_path']
        self.task_name = config['SETTINGS']['task_name']
       
        self.start_time = config['SETTINGS']['starttime']
        self.end_time = config['SETTINGS']['endtime']
        self.staid = config['SETTINGS']['aws_site']
        self.mosaic_range = int(config['SETTINGS']['mosaic_range'])
        
        self.pic_format = config['SETTINGS']['pic_format']
        self.pic_dpi = int(config['SETTINGS']['pic_dpi'])
        
        self.time_range = "[%s,%s]"%(self.start_time, self.end_time)

        # self.tlogp_time = '20230731000000'
        # 自动计算探空时间，用起始日期08时的探空
        self.tlogp_time = self.start_time[0:8] + '000000'

        self.outpath_tlogp = self.outpath + os.sep + 'tlogp'
        self.outpath_sta = self.outpath + os.sep + 'VPR'
        self.outpath_pic = self.outpath + os.sep + 'VPR'
        self.outpath_ref = self.outpath + os.sep + 'VPR'
        self.outname_sta= '%s.csv'%self.staid
        self.outname_ref = 'p3_vpr_%s_pyart.nc'%self.staid
        self.outpath_mosaic = self.outpath + os.sep + 'mosaic' + os.sep + self.task_name 
        
        if not os.path.exists(self.outpath):
            os.makedirs(self.outpath)
        if not os.path.exists(self.outpath_fmt):
            os.makedirs(self.outpath_fmt)
        if not os.path.exists(self.outpath_tlogp):
            os.makedirs(self.outpath_tlogp)
        if not os.path.exists(self.outpath_sta):
            os.makedirs(self.outpath_sta)
        if not os.path.exists(self.outpath_pic):
            os.makedirs(self.outpath_pic)
        if not os.path.exists(self.outpath_ref):
            os.makedirs(self.outpath_ref) 
        if not os.path.exists(self.outpath_mosaic):
            os.makedirs(self.outpath_mosaic)   

        self.colorfile = COLOR_FILE
        self.radarfile = RADAR_FILE
        self.base_mosaic_inifile = config['SETTINGS']['base_mosaic_inifile']
        self.fontfile = FONT_FILE
        self.data_code_radar_api = config['SETTINGS']['data_code_radar_api']
        self.data_code_aws_api = config['SETTINGS']['data_code_aws_api']

        # 默认拼图文件不可用
        self.bmosaic_vilid = False
    
    # 第一步，从天擎下载自动站数据
    def get_aws_data(self,):
        # 读取自动站雨量数据
        # A7606, A7607, A7055, A7617 重庆
        # # 54501 斋堂, A1067 丰台千灵山 A1254 大兴庞各庄
    
        print("entering get_aws_data function")
        elements='Station_Name,Station_Id_C,Station_Id_d,lat,lon,Datetime,PRE,PRE_1h'

        data_code =  self.data_code_aws_api

        # 读取数据
        data = cmadaas_obs_by_time_range_and_id(time_range=self.time_range, data_code = data_code,elements=elements,sta_ids = self.staid)
        if data is None:
            print('required data is None, please check the site number!')
            return False

        self.sta_lat = data['lat'][0]
        self.sta_lon = data['lon'][0]
        self.sta_name = data['Station_Name'][0]

        # data.to_csv(outpath_sta + os.sep + outname_sta,index=False)
        data2 = data.set_index('Datetime')
        # data2['PRE_1h'][0]=0


        diff_rain =[]
        diff_rain.append(0)


        # PRE_1h表示当前小时的累计雨量
        # pre_5min表示当前5分钟的累计雨量
        # pre_1min表示当前1分钟的累计雨量
        # accsum表示从起始时刻到当前时刻的累计雨量
        # 时间均为UTC时间

        # diff()为求差，mode()为获取众数
        # steps 为数据的时间间隔，单位为分钟
        steps = int(data['Datetime'].diff().mode()[0].seconds/60)
        difrain_name = 'pre_%dmin'%steps
        for nn in np.arange(0,data2.shape[0]-1):
            
            if data2['PRE_1h'][nn+1] >= data2['PRE_1h'][nn]:
                diff_rain.append(round(data2['PRE_1h'][nn+1]-data2['PRE_1h'][nn],2))
            else:
                diff_rain.append(round(data2['PRE_1h'][nn+1],2))
        newpd = pd.DataFrame(diff_rain,columns=[difrain_name],index=data2.index)
        # 如果间隔是1分钟，那么还需要求出5分钟的雨量
        if steps == 1:
            newpd['pre_5min'] = newpd[difrain_name].rolling(5).sum()
            newpd['pre_5min'] = newpd['pre_5min'].round(2)
        else:
            pass
        newindex = [tt for tt in newpd.index if tt.minute%5==0]
        newpd['accsum'] = newpd[difrain_name].cumsum()
        newpd['accsum'] = newpd['accsum'].round(2)
        newpd['PRE_1h'] = data2['PRE_1h']
        newpd['PRE'] = data2['PRE']
        newpd['lat'] = data2['lat']
        newpd['lon'] = data2['lon']
        newpd['staname']=data2['Station_Name']
        newpd.to_csv(self.outpath_sta + os.sep + self.outname_sta,index=True,encoding='gbk')
        return True

    def get_tlogp_data(self,):
        # 第二步，从离自动站最近的探空站中获取探空数据，提取零度层高度和-20度层高度
        # 读取探空站信息，获取经纬度、站号等信息
        # tlogpinfo = pd.read_csv('../common/stationinfo_new/china_tlogp_stations_m3.txt',delimiter=r"\s+",skiprows=3,names=['stanum','lon','lat','alt','num2'])
        
        # tmpdata['stanum'].values[0] int
        # staid_tlogp = str(tmpdata['stanum'].values[0])
        
        print("entering get_tlogp_data function")
        tlogpdata = cmadaas_sounding_by_time(times=self.tlogp_time)
        allstas = np.unique(tlogpdata['Station_Id_C'])
        alllons=[]
        alllats=[]
        #获取所有站点的经纬度信息
        for ts in allstas:
            tmpd = tlogpdata[tlogpdata['Station_Id_C']==ts]
            curlat = tmpd['Lat'].values[0]
            curlon = tmpd['Lon'].values[0]
            alllons.append(curlon)
            alllats.append(curlat)

        x,y= geotrans.geographic_to_cartesian_aeqd(lon=alllons,lat=alllats,lon_0=self.sta_lon,lat_0=self.sta_lat)
        dis = [math.sqrt(pow(x[k],2) + pow(y[k],2))/1000 for k in range(len(x))]
        flag = np.array(dis) <= min(dis)+0.1
        tmpts = allstas[flag]
        tmpdata = tlogpdata[tlogpdata['Station_Id_C']==tmpts[0]]
        tmpdata = tmpdata.sort_values(by='PRS_HWC',ascending=False)
        tmpdata.dropna(subset=['PRS_HWC'],inplace=True)
        tmpdata.to_csv(self.outpath_tlogp + os.sep + 'tlogp_%s.csv'%tmpts[0],index=False)
        #查找离零度层和-20度层最近的高度
        flag = np.array(abs(tmpdata['TEM'])) <= abs(tmpdata['TEM']).min()+0.01 
        z0 = tmpdata['GPH'][flag].values[0]
        flag = np.array(abs(tmpdata['TEM']+20)) <= abs(tmpdata['TEM']+20).min()+0.01
        z20 = tmpdata['GPH'][flag].values[0]

        self.z0 = geopotential_to_height(z0*9.80665)
        self.z20 = geopotential_to_height(z20*9.80665)
        return True
    
    def get_fmt_data(self,):
        # 第三步，从天擎读取雷达数据 
        # 先检查是否已经存在拼图文件，而且拼图文件中的时间和范围能够覆盖当前自动站
        # 如果已经能满足要求，则不再下载雷达数据
        print("entering get_fmt_data function")
        
        startt = datetime.strptime(self.start_time,'%Y%m%d%H%M%S')
        endt   = datetime.strptime(self.end_time,'%Y%m%d%H%M%S')
        curt = startt
        allref=[]
        grd_height = None
        need_file_times=[]
        while curt <= endt:
            
            curname = 'mosaic_' + curt.strftime('%Y%m%d%H%M') + '.nc'
            if os.path.exists(self.outpath_mosaic + os.sep + curname):
                # print('file exist: %s'%(outpath_mosaic + os.sep + curname))
                
                ref = xr.open_dataset(self.outpath_mosaic + os.sep + curname)

                cent_lat = ref.origin_latitude.values[0]
                cent_lon = ref.origin_longitude.values[0]
                x,y= geotrans.geographic_to_cartesian_aeqd(lon=self.sta_lon,lat=self.sta_lat,lon_0=cent_lon,lat_0=cent_lat)
                grd_height = ref['z'].values
                reso_x = ref['x'][1]-ref['x'][0]
                reso_y = ref['y'][1]-ref['y'][0]
                xx = x[0]/reso_x
                yy = y[0]/reso_y
                xx = int(xx.round())
                yy = int(yy.round())
                radius = int((len(ref.x)-1)/2)

                if xx+radius >=0 and xx+radius < len(ref.x) and yy+radius >=0 and yy+radius < len(ref.y):
                    pass
                else:
                    # 表示当前拼图范围不能覆盖自动站，需要重新下载数据进行拼图
                    need_file_times.append(curt)
                ref.close()
            else:
                need_file_times.append(curt)
            curt += timedelta(minutes=6)
            
        if len(need_file_times) == 0:
            self.bmosaic_vilid = True
            return True
 
        # 如果不能满足要求，则下载雷达数据

        # 根据自动站经纬度，自动判断应该下载哪些雷达站的数据
        # 读取雷达站点信息，获取经纬度、站号等信息
        valid_range = 200
        radinfo = pd.read_csv(self.radarfile,header=0,index_col=0)

        x,y = geotrans.geographic_to_cartesian_aeqd(radinfo['经度'],radinfo['纬度'],self.sta_lon,self.sta_lat, R=6370997.0)
        # dis = math.sqrt(pow(x[0],2) + pow(y[0],2))
        dis = [math.sqrt(pow(x[k],2) + pow(y[k],2))/1000 for k in range(len(x))]
        flag = np.array(dis) <= valid_range
        # flag = np.where(dis<=valid_range)[0]
        tmpdata = radinfo.loc[flag]
        self.rdinfo = tmpdata.loc[:,['站名','纬度','经度']]
        # # 暂时去掉不以Z9开头的站点
        
        self.used_radars = self.rdinfo.index.to_list()

        for site in self.rdinfo.index:
            print(site)
            # 暂时去掉不以Z9开头的站点
            if not site.startswith('Z9'):
                print("暂时不支持不以Z9开头的站点！")
                self.used_radars.remove(site)
                continue
            
            # 从天擎下载该站号的雷达资料
            if len(need_file_times) == 0:
                pass
            else:
                tmp_starttime = min(need_file_times).strftime('%Y%m%d%H%M%S')
                tmp_endtime = max(need_file_times).strftime('%Y%m%d%H%M%S')
                time_range = '[ ' + tmp_starttime + ',' + tmp_endtime +  ']'
                cmadaas_radar_level2_by_timerange_and_id(radar_ids=site,time_range=time_range,outpath=self.outpath_fmt)
        
        return True

        
        
    # 第四步，制作区域三维拼图
    def make_mosaic(self,):
        # 修改配置文件
        print("entering make_mosaic function")
        config = configparser.ConfigParser()
        config.read(self.base_mosaic_inifile)
        config.set('ARCHIVE', 'input_path_archive', self.outpath_fmt) # 
        config.set('ARCHIVE', 'output_path_archive', self.outpath_mosaic) # 
        config.set('ARCHIVE', 'starttime', self.start_time[0:12]) # 
        config.set('ARCHIVE', 'endtime', self.end_time[0:12]) # 
        config.set('COMMON_SETTING', 'center_lon', '%.3f'%self.sta_lon) # 
        config.set('COMMON_SETTING', 'center_lat', '%.3f'%self.sta_lat) # 
        config.set('COMMON_SETTING', 'mosaic_range', '50') # 


        str_site=''
        for site in self.used_radars:
            str_site += site+','
        config.set('COMMON_SETTING', 'radars', str_site) #
        # 在当前脚本路径下，创建临时ini文件，其中包含任务名
        cur_ini = SCRIPT_DIR / ('make_mosaic_' + self.task_name + '.ini')
        config.write(open(cur_ini, 'w')) # 将修改后的配置写入文件

        _make_mosaic = MAKE_RADAR_MOSAIC(cur_ini)

        if not _make_mosaic.berror:
            if _make_mosaic.run_mode == 'archive':
                _make_mosaic.do_archive()
            else:
                print('run mode is not archive!')
        return True
    

    # 第五步：从三维拼图数据中获取格点的时间序列
    def make_vpr(self,):
        print("entering make_vpr function")
        data = pd.read_csv(self.outpath_sta+os.sep+self.outname_sta,index_col=0,encoding='gbk')
        sta_lat = data['lat'][0]
        sta_lon = data['lon'][0]
        sta_name = data['staname'][0]
        startt = datetime.strptime(self.start_time,'%Y%m%d%H%M%S')
        endt   = datetime.strptime(self.end_time,'%Y%m%d%H%M%S')
        curt = startt
        allref=[]
        grd_height = None
        file_times=[]
        while curt <= endt:
            
            curname = 'mosaic_' + curt.strftime('%Y%m%d%H%M') + '.nc'
            if not os.path.exists(self.outpath_mosaic + os.sep + curname):
                print('file not exist: %s'%(self.outpath_mosaic + os.sep + curname))
                curt += timedelta(minutes=6)
                continue
                
            ref = xr.open_dataset(self.outpath_mosaic + os.sep + curname)

            cent_lat = ref.origin_latitude.values[0]
            cent_lon = ref.origin_longitude.values[0]
            x,y= geotrans.geographic_to_cartesian_aeqd(lon=sta_lon,lat=sta_lat,lon_0=cent_lon,lat_0=cent_lat)
            grd_height = ref['z'].values
            reso_x = ref['x'][1]-ref['x'][0]
            reso_y = ref['y'][1]-ref['y'][0]
            xx = x[0]/reso_x
            yy = y[0]/reso_y
            xx = int(xx.round())
            yy = int(yy.round())
            radius = int((len(ref.x)-1)/2)
            ref2 = ref.isel(x=xx+radius,y=yy+radius)['reflectivity'].values
            # 对ref2进行线性插值
            tmpdf = pd.DataFrame(ref2[0],columns=['ref_raw'])
            tmpdf['ref_new'] = tmpdf['ref_raw'].interpolate(method='slinear')
            ref2 = tmpdf['ref_new'].values
            allref.append(ref2)
            file_times.append(curt)
            curt += timedelta(minutes=6)
            ref.close()
        allref = np.array(allref)
        # allref = allref.reshape(allref.shape[0],allref.shape[2])

        alldata = MA.masked_array(allref, mask=allref==-9999)
        xrdata = xr.Dataset({
                    'dbz':(['z', 'time'], alldata.T, {'long name':'time-height dBZ'})},
                    coords={'z':grd_height, 'time':file_times},
                    attrs={'lat':sta_lat, 'lon':sta_lon})

        xrdata.to_netcdf(self.outpath_ref + os.sep + self.outname_ref)
        print('ref data success!')
        return True
    
    # 第六步，绘制图形
    def draw_pic(self,):
        
        print("entering draw_pic function")
        oridf = pd.read_csv(self.outpath_sta+os.sep+self.outname_sta,encoding='gbk')#,index_col=0

        oridf['Datetime'] = pd.to_datetime(oridf['Datetime'], format="%Y-%m-%d %H:%M:%S")
        oridf = oridf.set_index('Datetime')
        newindex = [tt for tt in oridf.index if tt.minute%5==0]
        df = oridf.loc[newindex]
        # df.set_index('staname',inplace=True)
        df.reset_index(inplace=True)

        #将缺失的时间进行插值处理
        helper = pd.DataFrame({'Datetime': pd.date_range(start=df['Datetime'].min(), end=df['Datetime'].max(),freq='300s')})
        newdf = pd.merge(df, helper, on='Datetime', how='outer').sort_values('Datetime')
        newdf['accsum'] = newdf['accsum'].interpolate(method='linear')

        # ref_colorfile='gr2_colors/default_BR_PUP1.pal'
        # ref_colorfile = 'gr2_colors/BR_WDTB_Bright.pal'
        outdic= parse_pro(self.colorfile)
        cmap=outdic['cmap']
        norm=outdic['norm']
        units=outdic['units'] 
        # from pyart.graph import common
        # cmapname = 'pyart_NWSRef'
        # cmap = common.parse_cmap(cmapname, 'reflectivity')

        data = xr.open_dataset(self.outpath_ref + os.sep + self.outname_ref)

        data = data.rolling(z=2, time=2, min_periods=1, center=True).mean()

        font_path = self.fontfile 
        font_manager.fontManager.addfont(font_path)
        prop = font_manager.FontProperties(fname=font_path)

        plt.rcParams["font.size"] = 12
        plt.rcParams["font.sans-serif"] = prop.get_name()
        fig = plt.figure(figsize=(10,6))
        ax1 = fig.add_axes([0.1,0.18,0.8,0.3])#位置[左,下,右,上]
        ax3 = fig.add_axes([0.1,0.64,0.8,0.3])#位置[左,下,右,上]
        ax2 = ax3.twinx()
        ax1.minorticks_on()


        cb_ax = fig.add_axes([0.25, 0.08, 0.5, 0.03])

        grid_y,grid_x=np.meshgrid(np.arange(0,data.dbz.shape[1]),data.z.values,)
        pcm = ax1.pcolormesh(grid_y,grid_x,data.dbz,cmap=cmap,norm=norm)
        # pcm = ax1.pcolormesh(data.dbz,cmap=cmap,norm=norm)


        # flag = np.array(abs(data.z.values - z0)) <= abs(data.z.values - z0).min()+0.01 
        # z0h = ax1.plot(np.arange(0,data.dbz.shape[1]),np.ones(data.dbz.shape[1])*int(np.where(flag)[0][0]),'y--',
        #                linewidth=2,label='0度层高度')
        # flag = np.array(abs(data.z.values - z20)) <= abs(data.z.values - z20).min()+0.01 
        # z20h=ax1.plot(np.arange(0,data.dbz.shape[1]),np.ones(data.dbz.shape[1])*int(np.where(flag)[0][0]),'r--',
        #               linewidth=2,label='-20度层高度')
        z0h = ax1.plot(np.arange(0,data.dbz.shape[1]),np.ones(data.dbz.shape[1])*self.z0,'k--',linewidth=2,label='0度层高度')
        z20h=ax1.plot(np.arange(0,data.dbz.shape[1]),np.ones(data.dbz.shape[1])*self.z20,'r--',linewidth=2,label='-20度层高度')
        
        # leg = ax1.legend([z0h], ["零度层高度",], loc='upper right',title_fontproperties=prop)
        ax1.legend()
        ax1.tick_params(axis='x',which='minor',length=3,width=0.8,)
        ax1.tick_params(axis='x',which='major',length=5,width=1,)
        cb = plt.colorbar(mappable=pcm,cax=cb_ax,extend='both',ticks=np.arange(-10,85,5),orientation='horizontal')
        cb.set_label('反射率因子 dBZ',fontsize=12)
        ah1=ax2.plot(newdf['accsum'].values,'r',linewidth=2,label='累计雨量')
        ah2=ax3.bar(np.arange(0,newdf.shape[0]),newdf['pre_5min'],label='5分钟雨量')
        ax2.legend(loc='upper right')
        ax3.legend(loc='upper left')
        ax3.minorticks_on()
        ax3.tick_params(axis='x',which='minor',length=3,width=0.8,)
        ax3.tick_params(axis='x',which='major',length=5,width=1,)
        ax3.xaxis.set_minor_locator(MultipleLocator(1))
        ax1.set_xlim([0,data.dbz.shape[1]+1])
        ax2.set_xlim([0,newdf.shape[0]])
        ax1.grid('y')
        ax2.grid('y')
        # plt.xticks(fontsize=12)
        # plt.yticks(fontsize=12)
        # ax1.set_xlabel('Time(BJT, day hour:min)')
        ax1.set_ylabel('距离地面的高度(米)')
        ax1.set_title('单点回波强度时间-高度图（%.2f°N,%.2f°E）'%(data.lat,data.lon),fontsize=12)
        ax2.set_ylabel('累计雨量(毫米)',color='r')
        ax3.set_ylabel('5分钟雨量(毫米)',color=[59/255,117/255,175/255])

        tmpdate = pd.to_datetime(data.time)+ timedelta(hours=8)
        dates1 = [date.strftime('%H:%M') for date in tmpdate]
        dates1[0] = tmpdate.min().strftime('%H:%M \n%Y-%m-%d')

        datestr = dates1[::5]
        tmpd = tmpdate[::5]
        datestr[-1] = tmpd.max().strftime('%H:%M \n%Y-%m-%d')

        ax1.set_xticks(np.arange(0,data.time.shape[0],5))
        ax1.set_xticklabels(datestr)
        # ax1.set_yticks(np.arange(0,data.z.shape[0],5))
        # ax1.set_yticklabels(data.z[::5].values.astype('int'))
        # ax1.set_yticks(data.z[::5])
        # ax1.set_ylim([data.z.min(),data.z.max()])
        ax1.set_ylim([0,20000])
        ax1.set_yticks(np.arange(0,20000,2500))

        tmpdate = pd.to_datetime(newdf['Datetime'])+ timedelta(hours=8)
        dates2 = [date.strftime('%H:%M') for date in tmpdate]
        dates2[0] = tmpdate.min().strftime('%H:%M \n%Y-%m-%d')
        dates2[-1] = tmpdate.max().strftime('%H:%M \n%Y-%m-%d')
        ax2.set_xticks(np.arange(0,newdf.shape[0],6))

        ax2.set_xticklabels(dates2[::6])
        ax3.set_xlabel('时间(时:分，北京时)')
        ax2.set_title('5分钟雨量（柱）、自动站累积雨量（红实线）, 站号：%s,%s(%.2f°N,%.2f°E)'%(self.staid,self.sta_name,self.sta_lat,self.sta_lon),fontsize=12)
        # plt.figtext(0.73,0.02,'国家气象中心-天气预报技术研发室制作',fontsize=10)
        plt.savefig(self.outpath_pic + os.sep + self.outname_ref.replace('.nc','.%s'%self.pic_format),dpi=self.pic_dpi)#,dpi=600
        print(self.outpath_pic + os.sep + self.outname_ref.replace('.nc','.%s'%self.pic_format) + ' saved!')
        # plt.show()
        plt.close()
        return True
    
    # 一键生成VPR数据和自动站数据，并绘制图形
    def do_all(self,):

        if not self.get_aws_data():
            return False
        if not self.get_tlogp_data():
            return False
        if not self.get_fmt_data():
            return False
        
        # 如果拼图文件不可用，则重新制作拼图
        if not self.bmosaic_vilid:
            if not self.make_mosaic():
                return False
        else:
            print("拼图文件已经存在，跳过拼图制作步骤。")
            
        if not self.make_vpr():
            return False
        if not self.draw_pic():
            return False

if __name__ == "__main__":


    freeze_support()

    # sitelist=['G1174','G3522']
    # for st in sitelist:
    #     _construct_vpr = CONSTRUCT_VPR(inifilepath='construct_aws_refvpr_mainprog_%s.ini'%st)
    #     _construct_vpr.do_all()

    # 根据实际情况修改配置文件路径
    _construct_vpr = CONSTRUCT_VPR(inifilepath='/home/wjzhu/OneDrive/PythonCode/MyWork/metradar/metradar/project/make_vpr_aws/construct_aws_refvpr_mainprog.ini')
    _construct_vpr.do_all()

    
    
    

    
    
